package jp.co.argo21.nautica.ejbtest;
import static org.junit.Assert.*;

import java.sql.Timestamp;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import jp.co.argo21.nautica.workflow.sample.model.AppKind;
import jp.co.argo21.nautica.workflow.sample.model.AppTransaction;
import jp.co.argo21.nautica.workflow.sample.model.Application;
import jp.co.argo21.nautica.workflow.sample.model.dao.AppTransactionDAO;
import jp.co.argo21.nautica.workflow.sample.model.dao.ApplicationDAO;
import jp.co.argo21.nautica.workflow.sample.model.dao.DAOFactory;
import jp.co.argo21.nautica.workflow.sample.model.dao.OvertimeAppDAO;
import jp.co.argo21.nautica.workflow.sample.util.UniqueKeyGenerator;
import jp.co.argo21.nautica.workflow.soap.InvalidFilterException;
import jp.co.argo21.nautica.workflow.soap.InvalidSessionException;
import jp.co.argo21.nautica.workflow.soap.InvalidWorkItemException;
import jp.co.argo21.nautica.workflow.soap.OrganizationManager;
import jp.co.argo21.nautica.workflow.soap.OrganizationManager_Service;
import jp.co.argo21.nautica.workflow.soap.RoleManager;
import jp.co.argo21.nautica.workflow.soap.RoleManager_Service;
import jp.co.argo21.nautica.workflow.soap.UserManager;
import jp.co.argo21.nautica.workflow.soap.UserManager_Service;
import jp.co.argo21.nautica.workflow.soap.WorkItemHandler;
import jp.co.argo21.nautica.workflow.soap.WorkItemHandler_Service;
import jp.co.argo21.nautica.workflow.soap.WorkflowEngineHandler;
import jp.co.argo21.nautica.workflow.soap.WorkflowEngineHandler_Service;
import jp.co.argo21.nautica.workflow.soap.WorkflowServiceConnector;
import jp.co.argo21.nautica.workflow.soap.WorkflowServiceConnector_Service;
import jp.co.argo21.nautica.workflow.soap.message.Attribute;
import jp.co.argo21.nautica.workflow.soap.message.Filter;
import jp.co.argo21.nautica.workflow.soap.message.Organization;
import jp.co.argo21.nautica.workflow.soap.message.ProcessState;
import jp.co.argo21.nautica.workflow.soap.message.Role;
import jp.co.argo21.nautica.workflow.soap.message.User;
import jp.co.argo21.nautica.workflow.soap.message.WorkItem;

import org.junit.After;
import org.junit.Test;




public class XPDLTest {

	private static final int QUEUE_WAIT = 5000;
	private static final String APP_KIND_ID = "overtime";

	private String sessionId;
	private String appId;
	private String pid;
	private User user;
	private WorkflowServiceConnector wfServiceConnector;
	private WorkItemHandler wiHandler;

	/**
	 * 実施単位１；ログイン
	 */
	@Test
	public void unit01(){
		try{
			WorkflowServiceConnector_Service wsc = new WorkflowServiceConnector_Service();
			wfServiceConnector = wsc.getWorkflowServiceConnectorPort();
			sessionId = wfServiceConnector.connect("UID110101", "taro");
		} catch (Exception e){
			fail();
		}
	}


	/**
	 * 実施単位２：メニュー画面
	 */
	@Test
	public void unit02(){

		try {
			//試験１　ログイン
			unit01();

			//試験２　getUser
			UserManager_Service um = new UserManager_Service();
			UserManager userManager = um.getUserManagerPort();
			user = userManager.getUser("UID110101");

			//試験３　getOrganizationByUser
			OrganizationManager_Service om = new OrganizationManager_Service();
			OrganizationManager orgManager = om.getOrganizationManagerPort();
			List<Organization> org = orgManager.getOrganizationByUser(user);


			//確認１
			assertEquals("アルゴ", user.getLastname());
			//確認２
			assertEquals("企画営業部一課", org.get(0).getName());

		} catch(Exception e){
			e.printStackTrace();
			fail();
		}
		
	}


	/**
	 * 実施単位３：ログアウト
	 * @throws InvalidSessionException 
	 */
	@Test(expected= InvalidSessionException.class)
	public void unit03() throws InvalidSessionException {

		try {
			//試験１　ログイン
			unit01();

			WorkItemHandler_Service wih = new WorkItemHandler_Service();
			wiHandler = wih.getWorkItemHandlerPort();

			//試験２　ログアウト
			wfServiceConnector.disconnect(sessionId);
		} catch (Exception e) {
			e.printStackTrace();
			fail();
		}

		try {
			wiHandler.getWorkItems(sessionId, null);
			fail();
		} catch (InvalidSessionException e) {
			throw e;
		} catch (InvalidFilterException e) {
			e.printStackTrace();
			fail();
		}
	}


	/**
	 * 実施単位４：申請一覧画面
	 */
	@Test
	public void unit04(){

		try {
			//試験１　ユーザー取得
			unit02();

			//試験２　ロールを取得する。
			RoleManager_Service rm = new RoleManager_Service();
			RoleManager roleManager = rm.getRoleManagerPort();
			List<Role> roles = roleManager.getAttachedRoles2(user);

			//確認１
			Set<String> roleSet = new HashSet<String>();
			roleSet.add("企画営業部ロール");
			roleSet.add("一般社員ロール");

			for (Role tempRole : roles){		
				System.out.println(tempRole);
				assertTrue(roleSet.contains(tempRole.getName()));
			}
			assertEquals(2, roles.size());

		}catch (Exception e){
			e.printStackTrace();
			fail();
		}
	}


	/**
	 * 実施単位５　申請開始
	 */
	@Test
	public void unit05(){
		try {
			//試験１　ユーザ取得
			unit02();

			//試験２　createProcess

			//申請書番号を決定
			Date date = new Date();
			SimpleDateFormat formatter;
			StringBuffer number = new StringBuffer();
			formatter = new SimpleDateFormat("yyyyMMddHHmmss");
			number.append(formatter.format(date));
			appId = number.toString();
			//appId = UniqueKeyGenerator.generate(APP_KIND_ID);

			// 申請トランザクションテーブルにレコードを追加
			AppKind appKind = DAOFactory.getAppKindDAO().getAppKindFromID(APP_KIND_ID);
			String appKindID = appKind.getAppkindID();
			AppTransactionDAO apptransDao = DAOFactory.getAppTransactionDAO();
			System.out.println("appId: " + appId);
			System.out.println("appKindID: " + appKindID);
			System.out.println("userID: " + user.getId());
			apptransDao.insertForOverTime(appId, appKindID, user.getId());

			// 各申請のテーブルに申請書番号の空レコードを追加
			OvertimeAppDAO appKindDao 
			= (OvertimeAppDAO) DAOFactory.getApplicationDAOFromAppKindID(APP_KIND_ID);
			appKindDao.insertAppNo(appId);
			ApplicationDAO appDao = DAOFactory.getApplicationDAO();
			appDao.insert(appId, appKindID, user.getId());

			String pdid = "Overtime-0";

			WorkflowEngineHandler_Service weh = new WorkflowEngineHandler_Service();
			WorkflowEngineHandler wfEngineHandler = weh.getWorkflowEngineHandlerPort();

			pid = wfEngineHandler.createProcess(sessionId, pdid, appId);

			System.out.println("pid: " + pid);
			//試験３　assignProcessAttribute
			Attribute state = new Attribute();
			state.setName(Constants.STATE);
			state.setValue(StateConstants.APPLY_WAIT);
			state.setType(1);
			wfEngineHandler.assignProcessAttribute(sessionId, pid, state);

			Attribute appNo = new Attribute();
			appNo.setName(Constants.APPLICATION_NO);
			appNo.setValue(appId);
			appNo.setType(1);
			wfEngineHandler.assignProcessAttribute(sessionId, pid, appNo);

			//試験４　startProcess
			wfEngineHandler.startProcess(sessionId, pid);

			//確認１　ログが出力されることを確認する
			//確認２　createProcessは試験３が正常に実行されることで確認する

			//確認３
			Attribute stateCheck= wfEngineHandler.getProcessAttributeValue(sessionId, pid, Constants.STATE);
			assertEquals(StateConstants.APPLY_WAIT, stateCheck.getValue());
			Attribute appNoCheck = wfEngineHandler.getProcessAttributeValue(sessionId, pid, Constants.APPLICATION_NO);
			assertEquals(appId, appNoCheck.getValue());

			//確認４
			Set<String> stateString = new HashSet<String>();
			stateString.add("open.not_running.suspended");
			stateString.add("closed.completed");
			stateString.add("closed.terminated");
			stateString.add("closed.aborted");

			Thread.sleep(QUEUE_WAIT);

			List<ProcessState> proState = wfEngineHandler.getProcessStates(sessionId, pid, null);
			for(int i=0; i < proState.size(); i++){
				String getStateString = proState.get(i).getStateString();
				assertTrue(getStateString.contains(getStateString));
			}
			assertEquals(4, proState.size());
		} catch (Exception e) {
			e.printStackTrace();
			fail();
		} 
	}


	/**
	 * 実施単位６　申請キャンセル
	 */
	@Test
	public void unit06(){
		try {
			//試験１　ログイン
			//試験２　実施単位５を実行
			unit05();

			//試験３　プロセス変数[申請書番号]を空文字に設定
			WorkItemHandler_Service wih = new WorkItemHandler_Service();
			WorkItemHandler wiHandler = wih.getWorkItemHandlerPort();
			System.out.println("afterappId: " + appId);
			List<Filter> appNoFilter = new ArrayList<Filter>();
			Filter filter = new Filter();
			filter.setFilterName(Constants.APPLICATION_NO);
			filter.setFilterString(appId);
			filter.setType(5003);
			filter.setComparison(1);
			appNoFilter.add(filter);
			List<WorkItem> workItems = wiHandler.getWorkItems(sessionId, appNoFilter);
			System.out.println("workItems.length: " + workItems.size());
			String wid = workItems.get(0).getID();

			Attribute appNoCancel = new Attribute();
			appNoCancel.setName(Constants.APPLICATION_NO);
			appNoCancel.setValue("");
			appNoCancel.setType(1);
			wiHandler.assignWorkItemAttribute(sessionId, wid, appNoCancel);

			//試験４　completeWorkItem
			wiHandler.completeWorkItem(sessionId, wid);
			
			WorkflowEngineHandler_Service weh = new WorkflowEngineHandler_Service();
			WorkflowEngineHandler wfEngineHandler = weh.getWorkflowEngineHandlerPort();
			
			//確認１
			Thread.sleep(QUEUE_WAIT);
			Attribute appIdCheck = wfEngineHandler.getProcessAttributeValue(sessionId, pid, Constants.APPLICATION_NO);
			assertEquals("", appIdCheck.getValue());
			//確認２
			/*
			try{
				wiHandler.completeWorkItem(sessionId, wid);
				fail();
			} catch (InvalidWorkItemException e){
				throw e;
			}
			*/
			//確認３
			List<ProcessState> proState = wfEngineHandler.getProcessStates(sessionId, pid, null);
			assertEquals(0, proState.size());

		}catch(Exception e){
			e.printStackTrace();
			fail();
		}
	}

	/**
	 * 実施単位７　申請内容入力待ち申請一覧取得
	 */
	@Test
	public void unit07(){
		try{
			//準備作業
			//試験１　ログイン
			unit05();
			
			WorkItemHandler_Service wih = new WorkItemHandler_Service();
			WorkItemHandler wiHandler = wih.getWorkItemHandlerPort();
			WorkflowEngineHandler_Service weh = new WorkflowEngineHandler_Service();
			WorkflowEngineHandler wfEngineHandler = weh.getWorkflowEngineHandlerPort();
			//試験２　getWorkItems
			List<Filter> state = new ArrayList<Filter>();
			Filter filter = new Filter();
			filter.setFilterName(Constants.STATE);
			filter.setComparison(1);
			filter.setFilterString(StateConstants.APPLY_WAIT);
			filter.setType(5003);
			state.add(filter);
			List<WorkItem> items = wiHandler.getWorkItems(sessionId, state);

			//試験３　getWorkItemAttributeValue
		
			//確認１
			assertEquals(pid, items.get(items.size()-1).getProcessID());
			
			Attribute attr 
			= wfEngineHandler.getProcessAttributeValue(sessionId, pid, Constants.APPLICATION_NO);
			
			//確認２
			assertEquals(appId, attr.getValue());
			
		} catch (Exception e){
			e.printStackTrace();
			fail();
		}
	}


	/**
	 * 実施単位８　申請
	 */
	@Test
	public void unit08(){
		try{
			//試験１　ログイン
			//試験２　実施単位５
			unit05();

			//試験３　申請トランザクションテーブルの[承認ブロック番号]"0"の[状態]を"申請"に、[対象日]を申請日にする
			//試験４　申請トランザクションテーブルの[承認ブロック番号]"1"の[状態]を"待ち"に変更する。
			Application app = new Application();
			app.setDate(new Timestamp(System.currentTimeMillis()));
			AppTransactionDAO appTransDao = DAOFactory.getAppTransactionDAO();
			AppTransaction[] appTranses = appTransDao.getAppTransactionsByAppID(appId);

			for (int i=0; i<appTranses.length; i++) {
				AppTransaction appTrans = appTranses[i];
				int approveAccountID = appTrans.getApproveAccountID();
				if (approveAccountID == 0) {
					appTrans.setAppStatus(StateConstants.STATUS_TRANS_APPLY);
					appTrans.setAppUserID(user.getId());
					appTrans.setDate(app.getDate());	//日付をどここから取ってくればOK
					appTransDao.updateAppTransaction(appTrans);
				} else if (approveAccountID == 1) {
					appTrans.setAppStatus(StateConstants.STATUS_TRANS_WAIT);
					appTransDao.updateStatus(appTrans);
				} else if (approveAccountID == 2 || approveAccountID ==3){
					appTrans.setAppStatus(StateConstants.STATUS_TRANS_NOT_REACH);
					appTransDao.updateStatus(appTrans);
				}
			}

			WorkItemHandler_Service wih = new WorkItemHandler_Service();
			WorkItemHandler wiHandler = wih.getWorkItemHandlerPort();
			List<Filter> stateFilter = new ArrayList<Filter>();
			Filter filter = new Filter();
			filter.setFilterName(Constants.STATE);
			filter.setFilterString(StateConstants.APPLY_WAIT);
			filter.setType(5003);
			filter.setComparison(1);
			stateFilter.add(filter);
			List<WorkItem> items = wiHandler.getWorkItems(sessionId, stateFilter);
			System.out.println("Items.length: " + items.size());
			String wid = items.get(items.size()-1).getID();

			//試験５　assignWorkItemAttribute
			Attribute appDate = new Attribute();
			appDate.setName(Constants.APPLY_DATE);
			appDate.setValue(app.getDate().toString());
			appDate.setType(1);
			wiHandler.assignWorkItemAttribute(sessionId, wid, appDate);

			Attribute state = new Attribute();
			state.setName(Constants.STATE);
			state.setValue(StateConstants.APPROVAL_WAIT);
			state.setType(1);
			wiHandler.assignWorkItemAttribute(sessionId, wid, state); 

			//試験６　completeWorkItem
			wiHandler.completeWorkItem(sessionId, wid);
			
			WorkflowEngineHandler_Service weh = new WorkflowEngineHandler_Service();
			WorkflowEngineHandler wfEngineHandler = weh.getWorkflowEngineHandlerPort();

			//確認１
			Thread.sleep(QUEUE_WAIT);
			Attribute appDateCheck = wfEngineHandler.getProcessAttributeValue(sessionId, pid, Constants.APPLY_DATE);
			assertEquals(appDate.getValue(), appDateCheck.getValue());
			Attribute stateCheck = wfEngineHandler.getProcessAttributeValue(sessionId, pid, Constants.STATE);
			assertEquals(StateConstants.APPROVAL_WAIT, stateCheck.getValue());

			//確認２
			/*
			try{
				wiHandler.completeWorkItem(sessionId, wid);
				fail();
			} catch (InvalidWorkItemException e){

			}
			*/
		} catch (Exception e){
			e.printStackTrace();
			fail();
		}
	}


	/**
	 * 実施単位９　承認状況一覧
	 */
	@Test
	public void unit09(){
		try{
			//試験１　ログイン
			unit08();

			//試験　getProcesses
			List appIDList = new ArrayList();
			// ユーザがフローに参加している申請の申請書番号を取得
			AppTransaction[] apptsByUser = DAOFactory.getAppTransactionDAO().getAppTransactionsByUserID(user.getId());
			addAppID(appIDList, apptsByUser);

			// ユーザのロールがフローに含まれる申請の申請書番号を取得
			RoleManager_Service rm = new RoleManager_Service();
			RoleManager roleManager = rm.getRoleManagerPort();
			List<Role> roles = roleManager.getAttachedRoles2(user);
			AppTransactionDAO apptDAO = DAOFactory.getAppTransactionDAO();
			for (int i = 0; i < roles.size(); i++) {
				AppTransaction[] apptsByRole = apptDAO.getAppTransactionsByRoleID(roles.get(i).getId());
				addAppID(appIDList, apptsByRole);
			}

			// 取得した申請書番号をもとに、プロセスが実行中である申請を取得

			WorkflowEngineHandler_Service weh = new WorkflowEngineHandler_Service();
			WorkflowEngineHandler wfEngineHandler = weh.getWorkflowEngineHandlerPort();
			List<List<jp.co.argo21.nautica.workflow.soap.message.Process>> appList = new ArrayList<List<jp.co.argo21.nautica.workflow.soap.message.Process>>();
			ApplicationDAO appDAO = DAOFactory.getApplicationDAO();
			Iterator ite = appIDList.iterator();
			while (ite.hasNext()) {
				String appID = (String) ite.next();

				// フィルターをセット
				List<Filter> groupFilter = new ArrayList<Filter>();
				Filter nameFilter = new Filter();
				nameFilter.setFilterName(appID);
				nameFilter.setType(5001);
				nameFilter.setComparison(1);
				groupFilter.add(nameFilter);
				Filter stateFilter = new Filter();
				stateFilter.setFilterName("ProcessState");
				stateFilter.setFilterString("open.running");
				stateFilter.setType(5002);
				groupFilter.add(stateFilter);
				List<jp.co.argo21.nautica.workflow.soap.message.Process> processes = wfEngineHandler.getProcesses(sessionId, groupFilter);
				appList.add(processes);
			}

			//確認１
			for (List<jp.co.argo21.nautica.workflow.soap.message.Process> proc : appList){
				for(int i = 0; i < proc.size(); i++){
					assertEquals(pid, proc.get(i));
				}

			}

		} catch (Exception e){
			e.printStackTrace();
			fail();
		}
	}


	/**
	 * 実施単位１０　承認待ち申請一覧取得
	 */
	@Test
	public void unit10(){
		try{
			//準備
			unit08();
			accessClose();

			//試験１　ログイン
			WorkflowServiceConnector_Service wsc = new WorkflowServiceConnector_Service();
			wfServiceConnector = wsc.getWorkflowServiceConnectorPort();
			sessionId = wfServiceConnector.connect("UID110001", "jiro");

			WorkItemHandler_Service wih = new WorkItemHandler_Service();
			WorkItemHandler wiHandler = wih.getWorkItemHandlerPort();
			
			//試験２　getWorkItems
			List<Filter> state = new ArrayList<Filter>();
			Filter filter = new Filter();
			filter.setFilterName(Constants.STATE);
			filter.setComparison(1);
			filter.setFilterString(StateConstants.APPROVAL_WAIT);
			filter.setType(5003);
			state.add(filter);
			List<WorkItem> items = wiHandler.getWorkItems(sessionId, state);
			
			//試験３　getWorkItemAttributeValue
	
			//確認１
			String checkPid = items.get(items.size()-1).getProcessID();
			assertEquals(pid, checkPid);

			WorkflowEngineHandler_Service weh = new WorkflowEngineHandler_Service();
			WorkflowEngineHandler wfEngineHandler = weh.getWorkflowEngineHandlerPort();
			String appIdCheck = wfEngineHandler.getProcessAttributeValue(sessionId, pid, Constants.APPLICATION_NO).getValue();
				
			//確認２
			assertEquals(appId, appIdCheck);
			
		} catch (Exception e){
			e.printStackTrace();
			fail();
		}
	}		
	
	/**
	 * 実施単位１１　差戻し
	 */
	@Test
	public void unit11(){
		try{
			//準備
			unit08();
			accessClose();

			//試験１　ログイン
			WorkflowServiceConnector_Service wsc = new WorkflowServiceConnector_Service();
			wfServiceConnector = wsc.getWorkflowServiceConnectorPort();
			sessionId = wfServiceConnector.connect("UID110001", "jiro");
			
			WorkItemHandler_Service wih = new WorkItemHandler_Service();
			WorkItemHandler wiHandler = wih.getWorkItemHandlerPort();
			
			List<Filter> stateFilter = new ArrayList<Filter>();
			Filter filter = new Filter();
			filter.setFilterName(Constants.STATE);
			filter.setFilterString(StateConstants.APPROVAL_WAIT);
			filter.setType(5003);
			filter.setComparison(1);
			stateFilter.add(filter);
			List<WorkItem> items = wiHandler.getWorkItems(sessionId, stateFilter);
			System.out.println("Items: " + items.size());
			String wid = items.get(items.size()-1).getID();
			
			//試験２　申請トランザクションテーブルの[状態]が"待ち"になっている最小の[承認ブロック番号]を取得する
			AppTransactionDAO appTransDao = DAOFactory.getAppTransactionDAO();
			AppTransaction[] appTranses = appTransDao.getAppTransactionsByAppID(appId);
			AppTransaction appTrans = null;
			for (int i = 0; i < appTranses.length; i++) {
				String status = appTranses[i].getAppStatus();
				if ((StateConstants.STATUS_TRANS_WAIT).equals(status)) {
					appTrans = appTranses[i];
					break;
				}
			}

			//試験３　申請書番号と取得した承認ブロック番号で申請トランザクションテーブルを更新
			if (appTrans != null) {
				appTrans.setAppStatus(StateConstants.STATUS_TRANS_RETRY);
				appTrans.setAppUserID("UID110001");
				appTrans.setComment("retry!");
				appTrans.setDate(new Timestamp(Calendar.getInstance().getTimeInMillis()));
				appTransDao.updateAppTransaction(appTrans);

				// 申請者の[状態]を”待ち”に変更
				AppTransaction applyTrans = appTransDao.getAppTransactionByAppIDBlockID(appId, 0);
				applyTrans.setAppStatus(StateConstants.STATUS_TRANS_WAIT);
				appTransDao.updateAppTransaction(applyTrans);
			}

			//試験４　assignWorkItemAttribute
			
			Attribute appState = new Attribute();
			appState.setName(Constants.STATE);
			appState.setValue(StateConstants.APPLY_WAIT);
			appState.setType(1);
			wiHandler.assignWorkItemAttribute(sessionId, wid, appState);
			
			Attribute appResult = new Attribute();
			appResult.setName(Constants.APPROVAL_RESULT);
			appResult.setValue(Constants.RETRY);
			appResult.setType(1);
			wiHandler.assignWorkItemAttribute(sessionId, wid, appResult);

			//試験５　completeWorkItem
			wiHandler.completeWorkItem(sessionId, wid);

			WorkflowEngineHandler_Service weh = new WorkflowEngineHandler_Service();
			WorkflowEngineHandler wfEngineHandler = weh.getWorkflowEngineHandlerPort();
			
			//確認１
			Thread.sleep(QUEUE_WAIT);
			Attribute appStateCheck = wfEngineHandler.getProcessAttributeValue(sessionId, pid, Constants.STATE);
			assertEquals(StateConstants.APPLY_WAIT, appStateCheck.getValue());
			Attribute appResultCheck = wfEngineHandler.getProcessAttributeValue(sessionId, pid, Constants.APPROVAL_RESULT);
			assertEquals(Constants.RETRY, appResultCheck.getValue());

			//確認２
			/*
			try{
				wiHandler.completeWorkItem(sessionId, wid);
				fail();
			} catch (InvalidWorkItemException e){
				throw e;
			}
			*/
		} catch (Exception e){
			e.printStackTrace();
			fail();
		}
	}


	/**
	 * 実施単位１２＿１　承認処理（課長承認）
	 */
	@Test
	public void unit12_1(){
		try{
			//準備)
			unit08();
			accessClose();

			//試験１　ログイン
			WorkflowServiceConnector_Service wsc = new WorkflowServiceConnector_Service();
			wfServiceConnector = wsc.getWorkflowServiceConnectorPort();
			sessionId = wfServiceConnector.connect("UID110001", "jiro");
			
			WorkItemHandler_Service wih = new WorkItemHandler_Service();
			WorkItemHandler wiHandler = wih.getWorkItemHandlerPort();
			
			List<Filter> stateFilter = new ArrayList<Filter>();
			Filter filter = new Filter();
			filter.setFilterName(Constants.STATE);
			filter.setFilterString(StateConstants.APPROVAL_WAIT);
			filter.setType(5003);
			filter.setComparison(1);
			stateFilter.add(filter);
			List<WorkItem> items = wiHandler.getWorkItems(sessionId, stateFilter);
			String wid = items.get(items.size()-1).getID();
			
			//試験２　申請トランザクションテーブルの[状態]が"待ち"になっている最小の[承認ブロック番号]を取得する
			AppTransactionDAO appTransDao = DAOFactory.getAppTransactionDAO();
			AppTransaction[] appTranses = appTransDao.getAppTransactionsByAppID(appId);
			AppTransaction appTrans = null;
			AppTransaction nextAppTrans = null;
			for (int i = 0; i < appTranses.length; i++) {
				String status = appTranses[i].getAppStatus();
				if ((StateConstants.STATUS_TRANS_WAIT).equals(status)) {
					appTrans = appTranses[i];
					if ( i < appTranses.length-1) {
						nextAppTrans = appTranses[i+1];
					}
					break;
				}
			}

			//試験３　申請書番号と取得した承認ブロック番号で申請トランザクションテーブルを更新
			if (appTrans != null) {
				appTrans.setAppStatus(StateConstants.STATUS_TRANS_APPROVE);
				appTrans.setAppUserID("UID110001");
				appTrans.setComment("OK!");
				appTrans.setDate(new Timestamp(Calendar.getInstance().getTimeInMillis()));
				appTransDao.updateAppTransaction(appTrans);
			}

			//試験４　assignWorkItemAttribute
			// 申請トランザクションテーブルの更新した次のレコードの
			// [状態]を"WAIT"に更新
			int approveAccount = 4;
			if (nextAppTrans != null && nextAppTrans.getApproveAccountID() < approveAccount) {
				nextAppTrans.setAppStatus(StateConstants.STATUS_TRANS_WAIT);
				appTransDao.updateStatus(nextAppTrans);
			}

			// 作業項目のプロセス変数[承認結果]を"ACK"に設定
			Attribute appResult = new Attribute();
			appResult.setName(Constants.APPROVAL_RESULT);
			appResult.setValue(Constants.ACK);
			appResult.setType(1);
			wiHandler.assignWorkItemAttribute(sessionId, wid, appResult);

			// 最終承認者の場合、プロセス変数[状態]を"CONFIRM_WAIT"に更新
			int approveCount = 4;
			if (approveCount == appTrans.getApproveAccountID()+1) {
				Attribute state = new Attribute();
				state.setName(Constants.STATE);
				state.setValue(StateConstants.CONFIRM_WAIT);
				state.setType(1);
				wiHandler.assignWorkItemAttribute(sessionId, wid, state);
			}

			//試験５　completeWorkItem
			wiHandler.completeWorkItem(sessionId, wid);

			WorkflowEngineHandler_Service weh = new WorkflowEngineHandler_Service();
			WorkflowEngineHandler wfEngineHandler = weh.getWorkflowEngineHandlerPort();

			//確認１
			Thread.sleep(QUEUE_WAIT);
			Attribute appStateCheck = wfEngineHandler.getProcessAttributeValue(sessionId, pid, Constants.STATE);
			assertEquals(StateConstants.APPROVAL_WAIT, appStateCheck.getValue());
			Attribute appResultCheck = wfEngineHandler.getProcessAttributeValue(sessionId, pid, Constants.APPROVAL_RESULT);
			assertEquals(Constants.ACK, appResultCheck.getValue());

			//確認２
			/*
			try{
				wiHandler.completeWorkItem(sessionId, wid);
				fail();
			} catch (InvalidWorkItemException e){
				throw e;
			}
			*/
		} catch (Exception e){
			e.printStackTrace();
			fail();
		}

	}


	/**
	 * 実施単位１２＿２　承認処理（部長承認）
	 */
	@Test
	public void unit12_2(){
		try{
			//準備
			unit12_1();
			accessClose();

			//試験１　ログイン
			WorkflowServiceConnector_Service wsc = new WorkflowServiceConnector_Service();
			wfServiceConnector = wsc.getWorkflowServiceConnectorPort();
			sessionId = wfServiceConnector.connect("UID100001", "saburo");

			WorkItemHandler_Service wih = new WorkItemHandler_Service();
			WorkItemHandler wiHandler = wih.getWorkItemHandlerPort();
			
			List<Filter> stateFilter = new ArrayList<Filter>();
			Filter filter = new Filter();
			filter.setFilterName(Constants.STATE);
			filter.setFilterString(StateConstants.APPROVAL_WAIT);
			filter.setType(5003);
			filter.setComparison(1);
			stateFilter.add(filter);
			List<WorkItem> items = wiHandler.getWorkItems(sessionId, stateFilter);
			String wid = items.get(items.size()-1).getID();
			
			//試験２　申請トランザクションテーブルの[状態]が"待ち"になっている最小の[承認ブロック番号]を取得する
			AppTransactionDAO appTransDao = DAOFactory.getAppTransactionDAO();
			AppTransaction[] appTranses = appTransDao.getAppTransactionsByAppID(appId);
			AppTransaction appTrans = null;
			AppTransaction nextAppTrans = null;
			for (int i = 0; i < appTranses.length; i++) {
				String status = appTranses[i].getAppStatus();
				if ((StateConstants.STATUS_TRANS_WAIT).equals(status)) {
					appTrans = appTranses[i];
					if ( i < appTranses.length-1) {
						nextAppTrans = appTranses[i+1];
					}
					break;
				}
			}

			//試験３　申請書番号と取得した承認ブロック番号で申請トランザクションテーブルを更新
			if (appTrans != null) {
				appTrans.setAppStatus(StateConstants.STATUS_TRANS_APPROVE);
				appTrans.setAppUserID("UID100001");
				appTrans.setComment("OK!");
				appTrans.setDate(new Timestamp(Calendar.getInstance().getTimeInMillis()));
				appTransDao.updateAppTransaction(appTrans);
			}

			//試験４　assignWorkItemAttribute
			// 申請トランザクションテーブルの更新した次のレコードの
			// [状態]を"WAIT"に更新
			int approveAccount = 4;
			if (nextAppTrans != null && nextAppTrans.getApproveAccountID() < approveAccount) {
				nextAppTrans.setAppStatus(StateConstants.STATUS_TRANS_WAIT);
				appTransDao.updateStatus(nextAppTrans);
			}

			// 作業項目のプロセス変数[承認結果]を"ACK"に設定
			Attribute appResult = new Attribute();
			appResult.setName(Constants.APPROVAL_RESULT);
			appResult.setValue(Constants.ACK);
			appResult.setType(1);
			wiHandler.assignWorkItemAttribute(sessionId, wid, appResult);

			// 最終承認者の場合、プロセス変数[状態]を"CONFIRM_WAIT"に更新
			int approveCount = 4;
			if (approveCount == appTrans.getApproveAccountID()+1) {
				Attribute state = new Attribute();
				state.setName(Constants.STATE);
				state.setValue(StateConstants.CONFIRM_WAIT);
				state.setType(1);
				wiHandler.assignWorkItemAttribute(sessionId, wid, state);
			}

			//試験５　completeWorkItem
			wiHandler.completeWorkItem(sessionId, wid);

			WorkflowEngineHandler_Service weh = new WorkflowEngineHandler_Service();
			WorkflowEngineHandler wfEngineHandler = weh.getWorkflowEngineHandlerPort();

			//確認１
			Thread.sleep(QUEUE_WAIT);
			Attribute appStateCheck = wfEngineHandler.getProcessAttributeValue(sessionId, pid, Constants.STATE);
			assertEquals(StateConstants.APPROVAL_WAIT, appStateCheck.getValue());
			Attribute appResultCheck = wfEngineHandler.getProcessAttributeValue(sessionId, pid, Constants.APPROVAL_RESULT);
			assertEquals(Constants.ACK, appResultCheck.getValue());

			//確認２
			/*
			try{
				wiHandler.completeWorkItem(sessionId, wid);
				fail();
			} catch (InvalidWorkItemException e){
				throw e;
			}
			*/
		} catch (Exception e){
			e.printStackTrace();
			fail();
		}

	}



	/**
	 * 実施単位１２＿３　承認処理（人事部承認）
	 */
	@Test
	public void unit12_3(){
		try{
			//準備
			unit12_2();
			accessClose();

			//試験１　ログイン
			WorkflowServiceConnector_Service wsc = new WorkflowServiceConnector_Service();
			wfServiceConnector = wsc.getWorkflowServiceConnectorPort();
			sessionId = wfServiceConnector.connect("UID210101", "kazuko");

			WorkItemHandler_Service wih = new WorkItemHandler_Service();
			WorkItemHandler wiHandler = wih.getWorkItemHandlerPort();
			
			//試験１
			List<Filter> stateFilter = new ArrayList<Filter>();
			Filter filter = new Filter();
			filter.setFilterName(Constants.STATE);
			filter.setFilterString(StateConstants.APPROVAL_WAIT);
			filter.setType(5003);
			filter.setComparison(1);
			stateFilter.add(filter);
			List<WorkItem> items = wiHandler.getWorkItems(sessionId, stateFilter);
			String wid = items.get(items.size()-1).getID();
			
			//試験２　申請トランザクションテーブルの[状態]が"待ち"になっている最小の[承認ブロック番号]を取得する
			AppTransactionDAO appTransDao = DAOFactory.getAppTransactionDAO();
			AppTransaction[] appTranses = appTransDao.getAppTransactionsByAppID(appId);
			AppTransaction appTrans = null;
			AppTransaction nextAppTrans = null;
			for (int i = 0; i < appTranses.length; i++) {
				String status = appTranses[i].getAppStatus();
				if ((StateConstants.STATUS_TRANS_WAIT).equals(status)) {
					appTrans = appTranses[i];
					if ( i < appTranses.length-1) {
						nextAppTrans = appTranses[i+1];
					}
					break;
				}
			}

			//試験３　申請書番号と取得した承認ブロック番号で申請トランザクションテーブルを更新
			if (appTrans != null) {
				appTrans.setAppStatus(StateConstants.STATUS_TRANS_APPROVE);
				appTrans.setAppUserID("UID210101");
				appTrans.setComment("OK!");
				appTrans.setDate(new Timestamp(Calendar.getInstance().getTimeInMillis()));
				appTransDao.updateAppTransaction(appTrans);
			}

			//試験４　assignWorkItemAttribute
			// 申請トランザクションテーブルの更新した次のレコードの
			// [状態]を"WAIT"に更新
			int approveAccount = 4;
			if (nextAppTrans != null && nextAppTrans.getApproveAccountID() < approveAccount) {
				nextAppTrans.setAppStatus(StateConstants.STATUS_TRANS_WAIT);
				appTransDao.updateStatus(nextAppTrans);
			}

			// 作業項目のプロセス変数[承認結果]を"ACK"に設定
			Attribute appResult = new Attribute();
			appResult.setName(Constants.APPROVAL_RESULT);
			appResult.setValue(Constants.ACK);
			appResult.setType(1);
			wiHandler.assignWorkItemAttribute(sessionId, wid, appResult);

			// 最終承認者の場合、プロセス変数[状態]を"CONFIRM_WAIT"に更新
			int approveCount = 4;
			if (approveCount == appTrans.getApproveAccountID()+1) {
				Attribute state = new Attribute();
				state.setName(Constants.STATE);
				state.setValue(StateConstants.CONFIRM_WAIT);
				state.setType(1);
				wiHandler.assignWorkItemAttribute(sessionId, wid, state);
			}

			//試験５　completeWorkItem
			wiHandler.completeWorkItem(sessionId, wid);
			
			WorkflowEngineHandler_Service weh = new WorkflowEngineHandler_Service();
			WorkflowEngineHandler wfEngineHandler = weh.getWorkflowEngineHandlerPort();

			//確認１
			Thread.sleep(QUEUE_WAIT);
			Attribute appStateCheck = wfEngineHandler.getProcessAttributeValue(sessionId, pid, Constants.STATE);
			assertEquals(StateConstants.CONFIRM_WAIT, appStateCheck.getValue());
			Attribute appResultCheck = wfEngineHandler.getProcessAttributeValue(sessionId, pid, Constants.APPROVAL_RESULT);
			assertEquals(Constants.ACK, appResultCheck.getValue());
			//確認２
			/*
			try{
				wiHandler.completeWorkItem(sessionId, wid);
				fail();
			} catch (InvalidWorkItemException e){

			}
			*/
		} catch (Exception e){
			e.printStackTrace();
			fail();
		}

	}


	/**
	 * 実施単位１３　否認
	 */
	@Test
	public void unit13(){
		try{
			//準備
			unit08();
			accessClose();

			//試験１　ログイン
			WorkflowServiceConnector_Service wsc = new WorkflowServiceConnector_Service();
			wfServiceConnector = wsc.getWorkflowServiceConnectorPort();
			sessionId = wfServiceConnector.connect("UID110001", "jiro");

			WorkItemHandler_Service wih = new WorkItemHandler_Service();
			WorkItemHandler wiHandler = wih.getWorkItemHandlerPort();
			
			List<Filter> stateFilter = new ArrayList<Filter>();
			Filter filter = new Filter();
			filter.setFilterName(Constants.STATE);
			filter.setFilterString(StateConstants.APPROVAL_WAIT);
			filter.setType(5003);
			filter.setComparison(1);
			stateFilter.add(filter);
			List<WorkItem> items = wiHandler.getWorkItems(sessionId, stateFilter);
			String wid = items.get(items.size()-1).getID();
			
			//試験２　申請トランザクションテーブルの[状態]が"待ち"になっている最小の[承認ブロック番号]を取得する
			AppTransactionDAO appTransDao = DAOFactory.getAppTransactionDAO();
			AppTransaction[] appTranses = appTransDao.getAppTransactionsByAppID(appId);
			AppTransaction appTrans = null;
			for (int i = 0; i < appTranses.length; i++) {
				String status = appTranses[i].getAppStatus();
				if ((StateConstants.STATUS_TRANS_WAIT).equals(status)) {
					appTrans = appTranses[i];
					break;
				}
			}

			//試験３　申請書番号と取得した承認ブロック番号で申請トランザクションテーブルを更新
			if (appTrans != null) {
				appTrans.setAppStatus(StateConstants.STATUS_TRANS_DENIAL);
				appTrans.setAppUserID("UID110001");
				appTrans.setComment("could not approve!");
				appTrans.setDate(new Timestamp(Calendar.getInstance().getTimeInMillis()));
				appTransDao.updateAppTransaction(appTrans);
			}

			//試験４　assignWorkItemAttribute
			Attribute appResult = new Attribute();
			appResult.setName(Constants.APPROVAL_RESULT);
			appResult.setValue(Constants.NACK);
			appResult.setType(1);
			wiHandler.assignWorkItemAttribute(sessionId, wid, appResult);

			//試験５　completeWorkItem
			wiHandler.completeWorkItem(sessionId, wid);

			WorkflowEngineHandler_Service weh = new WorkflowEngineHandler_Service();
			WorkflowEngineHandler wfEngineHandler = weh.getWorkflowEngineHandlerPort();

			//確認１
			Thread.sleep(QUEUE_WAIT);
			Attribute appResultCheck = wfEngineHandler.getProcessAttributeValue(sessionId, pid, Constants.APPROVAL_RESULT);
			assertEquals(Constants.NACK, appResultCheck.getValue());

			//確認２
			/*
			try{
				wiHandler.completeWorkItem(sessionId, wid);
				fail();
			} catch (InvalidWorkItemException e){
				throw e;
			}
			*/
		} catch (Exception e){
			e.printStackTrace();
			fail();
		}
	}

	/**
	 * 実施単位１４　同報申請一覧
	 */
	@Test
	public void unit14(){
		try{
			//準備
			unit12_3();

			//試験１　ログイン
			WorkflowServiceConnector_Service wsc = new WorkflowServiceConnector_Service();
			wfServiceConnector = wsc.getWorkflowServiceConnectorPort();
			sessionId = wfServiceConnector.connect("UID200001", "yonko");

			WorkItemHandler_Service wih = new WorkItemHandler_Service();
			WorkItemHandler wiHandler = wih.getWorkItemHandlerPort();
			
			List<Filter> stateFilter = new ArrayList<Filter>();
			Filter filter = new Filter();
			filter.setFilterName(Constants.STATE);
			filter.setFilterString(StateConstants.CONFIRM_WAIT);
			filter.setType(5003);
			filter.setComparison(1);
			stateFilter.add(filter);
			List<WorkItem> items = wiHandler.getWorkItems(sessionId, stateFilter);
			
			//試験３　getWorkItemAttributeValue
			String checkPid = items.get(items.size()-1).getProcessID();
				
			//確認１
			assertEquals(pid, checkPid);
			
			WorkflowEngineHandler_Service weh = new WorkflowEngineHandler_Service();
			WorkflowEngineHandler wfEngineHandler = weh.getWorkflowEngineHandlerPort();
			String appIdCheck = wfEngineHandler.getProcessAttributeValue(sessionId, pid, Constants.APPLICATION_NO).getValue();
				
			//確認２
			assertEquals(appId, appIdCheck);

		} catch (Exception e){
			e.printStackTrace();
			fail();
		}

	}


	/**
	 * 実施単位１５　同報確認
	 */
	@Test
	public void unit15(){
		try{
			//準備
			unit12_3();

			//試験１　ログイン
			WorkflowServiceConnector_Service wsc = new WorkflowServiceConnector_Service();
			wfServiceConnector = wsc.getWorkflowServiceConnectorPort();
			sessionId = wfServiceConnector.connect("UID200001", "yonko");

			WorkItemHandler_Service wih = new WorkItemHandler_Service();
			WorkItemHandler wiHandler = wih.getWorkItemHandlerPort();
			
			List<Filter> stateFilter = new ArrayList<Filter>();
			Filter filter = new Filter();
			filter.setFilterName(Constants.STATE);
			filter.setFilterString(StateConstants.CONFIRM_WAIT);
			filter.setType(5003);
			filter.setComparison(1);
			stateFilter.add(filter);
			List<WorkItem> items = wiHandler.getWorkItems(sessionId, stateFilter);
			String wid = items.get(items.size()-1).getID();
			
			//getUser
			UserManager_Service um = new UserManager_Service();
			UserManager userManager = um.getUserManagerPort();
			user = userManager.getUser("UID200001");

			//試験２　getAttachedRoles
			RoleManager_Service rm = new RoleManager_Service();
			RoleManager roleManager = rm.getRoleManagerPort();
			List<Role> roles = roleManager.getAttachedRoles2(user);
			
			//試験３　assignWorkItemAttribute
			Attribute confirmState = new Attribute();
			confirmState.setType(3);        
			List list = new ArrayList();
			for (int i = 0; i < roles.size(); i++) {
				list.add(roles.get(i).getId());
			}        
			if (list.contains("JINJI_KACHO_ROLE")) {
				confirmState.setName("NOTIFICATION_JINJI_KACHO");
			} else if (list.contains("JINJI_BUCHO_ROLE")) {
				confirmState.setName("NOTIFICATION_JINJI_BUCHO");
			}        
			confirmState.setValue("true");
			wiHandler.assignWorkItemAttribute(sessionId, wid, confirmState);

			//試験４　completeWorkItem
			WorkflowEngineHandler_Service weh = new WorkflowEngineHandler_Service();
			WorkflowEngineHandler wfEngineHandler = weh.getWorkflowEngineHandlerPort();

			//確認１
			Thread.sleep(QUEUE_WAIT);
			Set<String> roleSet = new HashSet<String>();
			roleSet.add("JINJI_ROLE");
			roleSet.add("BUCHO_ROLE");
			roleSet.add("JINJI_BUCHO_ROLE");

			for (int i=0; i < roles.size(); i++){
				String tempRole = roles.get(i).getId();
				System.out.println(tempRole);
				assertTrue(roleSet.contains(tempRole));
			}
			assertEquals(3, roles.size());

			//確認２
			Attribute confirmStateCheck = wfEngineHandler.getProcessAttributeValue(
					sessionId, pid, "NOTIFICATION_JINJI_BUCHO");
			assertEquals("true", confirmStateCheck.getValue());

			//確認３
			/*
			try{
				wiHandler.completeWorkItem(sessionId, wid);
				fail();
			} catch (InvalidWorkItemException e){
				throw e;
			}
			*/
		} catch (Exception e){
			e.printStackTrace();
			fail();
		}
	}


	/**
	 * 実施単位９で使用。
	 * リストに含まれていない申請書番号を追加して返す。
	 * @param list　申請書番号のリスト
	 * @param appts　申請トランザクション
	 * @throws Exception　任意の例外
	 */
	private void addAppID(List list, AppTransaction[] appts) throws Exception {
		for (int i = 0; i < appts.length; i++) {
			String id = appts[i].getAppID();
			AppTransactionDAO appTransDao = DAOFactory.getAppTransactionDAO();
			AppTransaction appTrans = appTransDao.getAppTransactionByAppIDBlockID(id, 0);            
			if (!list.contains(id) && (StateConstants.STATUS_TRANS_APPLY).equals(appTrans.getAppStatus())) {
				list.add(id);
			}
		}
	}

	/**
	 * 開いたAccessorを閉じる。
	 */
	@After
	public void accessClose(){
		try {
			if (wfServiceConnector != null){
				//accessor.close();
				wfServiceConnector.disconnect(sessionId);
			}
		} catch (InvalidSessionException e) {
			fail();
		}
	}
}